/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_INTERFACE_PROTOCOL_H__
#define __TR_INTERFACE_PROTOCOL_H__

#include <sys/types.h>

#include "trbase.h"
#include "trio.h"

#include "tr/protocol.h"
#include "tr/proto_message.h"

typedef TR_ProtoMessage (* fptr_TR_protoCreateMessage)(void *, va_list *);
typedef TR_ProtoMessage (* fptr_TR_protoCreateRequest)(void *, va_list *);
typedef TR_ProtoMessage (* fptr_TR_protoCreateResponse)(void *, va_list *);
typedef TR_RemoteData   (* fptr_TR_protoCompose)(void *, TR_ProtoMessage);
typedef TR_RemoteData   (* fptr_TR_protoParse)(
		void *, TR_ProtoMessage, TR_RemoteData);

TR_INTERFACE(TR_Protocol) {
	TR_IFID;
	fptr_TR_protoCreateMessage  createMessage;
	fptr_TR_protoCreateRequest  createRequest;
	fptr_TR_protoCreateResponse createResponse;
	fptr_TR_protoParse          parse;
	fptr_TR_protoCompose        compose;
};

TR_ProtoMessage TR_vprotoCreateMessage(void *, TR_Socket, va_list*);
TR_ProtoMessage TR_protoCreateMessage(void *, TR_Socket, ...);
TR_ProtoMessage TR_vprotoCreateRequest(void *, TR_Socket, va_list*);
TR_ProtoMessage TR_protoCreateRequest(void *, TR_Socket, ...);
TR_ProtoMessage TR_vprotoCreateResponse(void *, TR_Socket, va_list*);
TR_ProtoMessage TR_protoCreateResponse(void *, TR_Socket, ...);
TR_RemoteData   TR_protoParse(void *, TR_ProtoMessage, TR_RemoteData);
TR_RemoteData   TR_protoCompose(void *, TR_ProtoMessage);

#endif // __TR_INTERFACE_PROTOCOL_H__

// vim: set ts=4 sw=4:
