/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_SERVER_H__
#define __TR_SERVER_H__

#include <sys/types.h>

#include "trbase.h"
#include "trevent.h"

#include "tr/comm_manager.h"
#include "tr/connector.h"
#include "tr/io_handler.h"
#include "tr/protocol_handler.h"
#include "tr/protocol.h"
#include "tr/interface/comm_manager.h"

TR_CLASS(TR_Server) {
	TR_CommManager     comm_manager;
	TR_EventDispatcher dispatcher;
	TR_Connector       connector;
	TR_IoHandler       io_handler;
	TR_ProtocolHandler protocol_handler;
};
TR_INSTANCE_INIT(TR_Server);
TR_CLASSVARS_DECL(TR_Server) {};

#define TR_serverAddEndpoint(srv, ep) \
	TR_commManagerAddEndpoint((srv)->comm_manager, (ep))

#define TR_serverAddHandler(srv, handler) \
	TR_eventDispatcherRegisterHandler((srv)->dispatcher, (handler))

void TR_serverBindTcp(TR_Server, const char *, int, TR_Protocol);
void TR_serverBindUdp(TR_Server, const char *, int, TR_Protocol);
void TR_serverStart(TR_Server, unsigned long);

#define TR_serverClassCleanup()                      \
	TR_eventHandlerClassCleanup(TR_ProtocolHandler); \
	TR_eventHandlerClassCleanup(TR_IoHandler);       \
	TR_eventHandlerClassCleanup(TR_Connector);       \
	TR_eventHandlerClassCleanup(TR_CommManagerPoll)

#endif // __TR_SERVER_H__

// vim: set ts=4 sw=4:
