/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define _GNU_SOURCE

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <stdarg.h>
#include <semaphore.h>
#include <unistd.h>
#include <fcntl.h>

#include "trbase.h"
#include "trio.h"

#include "tr/comm_worker.h"

static
int
commWorkerCtor(void * _this, va_list * params)
{
	TR_CommWorker      this = _this;
	TR_Socket          socket[2];

	this->shm = mmap(NULL, sizeof(s_TR_WorkerShm),
			PROT_READ|PROT_WRITE,
			MAP_ANONYMOUS|MAP_SHARED,
			0, 0);

	sem_init(&(this->shm->semaphore), 1, 1);
	TR_socketPair(socket, SOCK_DGRAM);

	/* TODO error handling... */
	switch (this->pid = fork()) {
		case -1:
			TR_delete(socket[0]);
			TR_delete(socket[1]);
			break;

		case 0:
			TR_delete(socket[0]);
			this->socket = socket[1];
			break;

		default:
			TR_delete(socket[1]);
			this->socket = socket[0];
			break;
	}

	return this->pid;
}

static void commWorkerDtor(void * _this) {
	TR_CommWorker this = _this;

	TR_delete(this->socket);
	sem_close(&(this->shm->semaphore));
	munmap(this->shm, sizeof(s_TR_WorkerShm));
	if (this->pid != 0) {
		int state;
		waitpid(this->pid, &state, 0);
	}
}

TR_INIT_IFACE(TR_Class, commWorkerCtor, commWorkerDtor, NULL);
TR_CREATE_CLASS(TR_CommWorker, NULL, NULL, TR_IF(TR_Class));

// vim: set ts=4 sw=4:
