/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdarg.h>
#include <stdint.h>

#include <sys/types.h>

#include "trbase.h"
#include "trio.h"
#include "trevent.h"

#include "tr/connect_entry_point.h"
#include "tr/interface/comm_end_point.h"

static
int
connEntryPointCtor(void * _this, va_list * params)
{
	TR_CommEndPoint this = _this;

	TR_PARENTCALL(TR_ConnEntryPoint, _this, TR_Class, ctor, params);

	TR_socketBind((TR_Socket)this->transport);
	TR_socketNonblock((TR_Socket)this->transport);

	return 0;
}

static
void
connEntryPointDtor(void * _this)
{
	TR_PARENTCALL(TR_ConnEntryPoint, _this, TR_Class, dtor);
}

static
void
connEntryPointCvInit(TR_class_ptr cls)
{
	TR_EVENT_CREATE(cls, TR_CET_EVENT_ACC_READY);
}

const char * TR_cetEventStrings[] = {
	"TR_CET_EVENT_ACC_READY"
};

intptr_t connEntryPoint_events[TR_CET_EVENT_MAX + 1];
TR_INIT_IFACE(TR_Class, connEntryPointCtor, connEntryPointDtor, NULL);
TR_INIT_IFACE(TR_CommEndPoint, NULL, NULL);
TR_CREATE_CLASS(
		TR_ConnEntryPoint,
		TR_CommEndPoint,
		connEntryPointCvInit,
		TR_IF(TR_Class),
		TR_IF(TR_CommEndPoint)) = {
	{{
		 TR_cetEventStrings,
		 TR_CET_EVENT_MAX + 1,
		 connEntryPoint_events
	}}
};

// vim: set ts=4 sw=4:
