/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdarg.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <semaphore.h>

#include "trbase.h"
#include "trio.h"
#include "trevent.h"

#include "tr/connection_getter.h"
#include "tr/connection.h"
#include "tr/protocol.h"
#include "tr/connect_entry_point.h"

static
int
connectionGetterCtor(void * _this, va_list * params)
{
	TR_ConnectionGetter this = _this;

	TR_PARENTCALL(TR_ConnectionGetter, _this, TR_Class, ctor, params);
	this->worker = va_arg(*params, TR_CommWorker);

	return 0;
}

static
void
connectionGetterDtor(void * _this)
{
	TR_PARENTCALL(TR_ConnectionGetter, _this, TR_Class, dtor);
}

static
TR_EventDone
connectionGetterAccept(void * _this, TR_Event event)
{
	int                 count      = 0;
	TR_ConnectionGetter this       = _this;
	TR_CommEndPoint     connection = (TR_CommEndPoint)event->subject;

	int handle = TR_socketGetFd((TR_Socket)connection->transport);

	while (handle != -1) {
		TR_Socket socket = TR_new(TR_Socket);

		memcpy(
				socket,
				&(this->worker->shm->socket),
				sizeof(struct c_TR_Socket));
		sem_post(&(this->worker->shm->semaphore));
		TR_socketHandle(socket) = handle;

		TR_socketNonblock(socket);
		TR_Connection new_con = TR_new(
				TR_Connection,
				socket,
				connection->protocol,
				CEP_DEFAULT_READ_SIZE);
		TR_eventHandlerIssueEvent(
				(TR_EventHandler)this,
				TR_eventSubjectEmit(
					(TR_EventSubject)new_con,
					TR_CON_EVENT_NEW_CON,
					NULL));
		if (++count > 100) break;
		handle = TR_socketGetFd((TR_Socket)connection->transport);
	}

	if (! handle) {
		TR_eventHandlerIssueEvent(
				(TR_EventHandler)this,
				TR_eventSubjectEmit(
					(TR_EventSubject)connection,
					TR_CEP_EVENT_READ_BLOCK,
					NULL));
	}

	return TR_EVENT_DONE;
}

static
void
connectorCvInit(TR_class_ptr cls)
{
    TR_EVENT_HANDLER_SET_METHOD(
            cls,
            TR_ConnEntryPoint,
            TR_CET_EVENT_ACC_READY,
            connectionGetterAccept);
}

TR_INIT_HANDLER(TR_ConnectionGetter);
TR_INIT_IFACE(TR_Class, connectionGetterCtor, connectionGetterDtor, NULL);
TR_CREATE_CLASS(
		TR_ConnectionGetter,
		TR_EventHandler,
		connectorCvInit,
		TR_IF(TR_Class)) = {
	{ TR_HANDLER_CVARS(TR_ConnectionGetter) }
};

// vim: set ts=4 sw=4:
