/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdarg.h>
#include <stdint.h>

#include <sys/types.h>

#include "trbase.h"
#include "trio.h"
#include "trevent.h"

#include "tr/connector.h"
#include "tr/connection.h"
#include "tr/protocol.h"
#include "tr/connect_entry_point.h"

static
int
connectorCtor(void * _this, va_list * params)
{
	TR_PARENTCALL(TR_Connector, _this, TR_Class, ctor, params);
	return 0;
}

static
void
connectorDtor(void * _this)
{
	TR_PARENTCALL(TR_Connector, _this, TR_Class, dtor);
}

static
TR_EventDone
connectorAccept(void * _this, TR_Event event)
{
	int             count      = 0;
	TR_Connector    this       = _this;
	TR_CommEndPoint connection = (TR_CommEndPoint)event->subject;
	TR_TcpSocket    socket     = TR_socketAccept(
			(TR_TcpSocket)connection->transport);

	while (socket) {
		TR_socketNonblock((TR_Socket)socket);
		TR_Connection new_con = TR_new(
				TR_Connection,
				socket,
				connection->protocol,
				CEP_DEFAULT_READ_SIZE);
		TR_eventHandlerIssueEvent(
				(TR_EventHandler)this,
				TR_eventSubjectEmit(
					(TR_EventSubject)new_con,
					TR_CON_EVENT_NEW_CON,
					NULL));
		/*
		 * TODO The break is fatal when using edge triggered events...
		 * so either make sure to only use level trigged events with
		 * the accept socket or remove the break...
		 * Currently I think the break can be removed...
		 *
		 * TODO Prevent malicious or broken clients from doing a connection
		 * bomb here... well, doing this might mean that the break needs to be
		 * here.
		 */
		if (++count > 100) break;
		socket = TR_socketAccept((TR_TcpSocket)connection->transport);
	}

	if (! socket) {
		TR_eventHandlerIssueEvent(
				(TR_EventHandler)this,
				TR_eventSubjectEmit(
					(TR_EventSubject)connection,
					TR_CEP_EVENT_READ_BLOCK,
					NULL));
	}

	return TR_EVENT_DONE;
}

static
void
connectorCvInit(TR_class_ptr cls)
{
    TR_EVENT_HANDLER_SET_METHOD(
            cls,
            TR_ConnEntryPoint,
            TR_CET_EVENT_ACC_READY,
            connectorAccept);
}

TR_INIT_HANDLER(TR_Connector);
TR_INIT_IFACE(TR_Class, connectorCtor, connectorDtor, NULL);
TR_CREATE_CLASS(
		TR_Connector,
		TR_EventHandler,
		connectorCvInit,
		TR_IF(TR_Class)) = {
	{ TR_HANDLER_CVARS(TR_Connector) }
};

// vim: set ts=4 sw=4:
