/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>

#include "trbase.h"
#include "trio.h"

#include "tr/interface/protocol.h"
#include "tr/proto_message.h"

TR_CREATE_INTERFACE(TR_Protocol, 5);

TR_ProtoMessage
TR_vprotoCreateMessage(void * _this, TR_Socket remote, va_list * params)
{
	TR_ProtoMessage callret;
	TR_RETCALL(_this, TR_Protocol, createMessage, callret, params);
	if (callret != NULL) {
		callret->remote = remote;
	}
	return callret;
}

TR_ProtoMessage
TR_protoCreateMessage(void * _this, TR_Socket remote, ...)
{
	TR_ProtoMessage callret;
	va_list         params;
	va_start(params, remote);
	callret = TR_vprotoCreateMessage(_this, remote, &params);
	va_end(params);
	return callret;
}

TR_ProtoMessage
TR_vprotoCreateRequest(void * _this, TR_Socket remote, va_list * params)
{
	TR_ProtoMessage callret;
	TR_RETCALL(_this, TR_Protocol, createRequest, callret, params);
	if (callret != NULL) {
		callret->remote = remote;
	}
	return callret;
}

TR_ProtoMessage
TR_protoCreateRequest(void * _this, TR_Socket remote, ...)
{
	TR_ProtoMessage callret;
	va_list         params;
	va_start(params, remote);
	callret = TR_vprotoCreateRequest(_this, remote, &params);
	va_end(params);
	return callret;
}

TR_ProtoMessage
TR_vprotoCreateResponse(void * _this, TR_Socket remote, va_list * params)
{
	TR_ProtoMessage callret;
	TR_RETCALL(_this, TR_Protocol, createResponse, callret, params);
	if (callret != NULL) {
		callret->remote = remote;
	}
	return callret;
}

TR_ProtoMessage
TR_protoCreateResponse(void * _this, TR_Socket remote, ...)
{
	TR_ProtoMessage callret;
	va_list         params;
	va_start(params, remote);
	callret = TR_vprotoCreateResponse(_this, remote, &params);
	va_end(params);
	return callret;
}

TR_RemoteData
TR_protoParse(void * _this, TR_ProtoMessage message, TR_RemoteData data)
{
	TR_RemoteData callret;
	TR_RETCALL(_this, TR_Protocol, parse, callret, message, data);
	return callret;
}

TR_RemoteData
TR_protoCompose(void * _this, TR_ProtoMessage message)
{
	TR_RemoteData callret;
	TR_RETCALL(_this, TR_Protocol, compose, callret, message);
	return callret;
}

// vim: set ts=4 sw=4:
